#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from .command import Command
from .display import Display
from .display import Resolution


class Controller:
    """Reads output of 'xrandr -q' to get all connected displays
    and their resolutions.
    Stores available displays.
    Handles user input to determine the respective xrandr commands.

    Provides the following methods:
        1) get_common_resolutions()
        2) print_status()
        3) get_built_in_display()
        4) reduce_output(display)
        5) clone_output()
        6) extend_output(diretion)
    """
    def __init__(self):
        """Get connected displays and their resolutions."""
        cmd = Command("xrandr -q")
        output = cmd.call(feedback=True).strip().split("\n")
        self._displays = []
        # Iterate over xrandr output
        for i, dispLine in enumerate(output):
            # Detect connected displays
            if " connected" in dispLine:
                displayName = dispLine.split(" ")[0]
                resolutions = []
                # Get available resolutions
                for resLine in output[i+1:]:
                    if resLine.startswith(" "):
                        resLine = resLine.strip(" ").split(" ")[0]
                        res = Resolution(resLine)
                        resolutions.append(res)
                    else:
                        break
                # Store display data
                disp = Display(displayName, resolutions)
                self._displays.append(disp)
        # Assume that the first display listed by xrandr is the main one.
        self._builtIn = self._displays[0]

    def get_common_resolutions(self):
        """Return list of resolutions shared by all connected displays."""
        # Create copy of first display
        dummy = Display("Dummy", self._builtIn.get_resolutions())
        # Iterate over remaining displays
        for disp in self._displays[1:]:
            # Create dummy display with intersection of all resolutions
            dummy = Display("Dummy", dummy & disp)
        return dummy.get_resolutions()

    def print_status(self, message):
        """Print final message about what the program just did
        before exiting.
        """
        print(message)
        sys.exit(0)

    def get_built_in_display(self):
        """Return built-in display."""
        return self._builtIn

    def reduce_output(self, disp):
        """Reduce the output to single display.
        The resolution is determined automatically by xrandr.
        """
        # Set output to display disp
        cmd = Command("xrandr")
        cmd += "--output"
        cmd += str(disp)
        cmd += "--auto"
        cmd += "--rotate normal"
        cmd += "--pos 0x0"
        # Turn off output of all other displays
        for disp in (x for x in self._displays if x != disp):
            cmd += "--output " + str(disp) + " --off"
        cmd.call()
        self.print_status("Reduced output.")

    def clone_output(self):
        """Clone output to all registered displays.
        Use the highest shared resolution.
        """
        if len(self._displays) < 2:
            print("Cannot clone: Not enough displays connected.")
            sys.exit(1)
        else:
            # Make sure each display gets the same resolution
            res = str(max(self.get_common_resolutions()))
            cmd = Command("xrandr")
            for disp in self._displays:
                cmd += "--output " + str(disp)
                cmd += "--mode " + res
                cmd += "--rotate normal"
                cmd += "--pos 0x0"
            cmd.call()
            self.print_status("Cloned output.")

    def extend_output(self, direction):
        """Extend the output to displays left or right
        of the built-in display.
        The resolution is determined automatically by xrandr.
        """
        if len(self._displays) < 2:
            print("Cannot extend output: Not enough displays connected.")
            sys.exit(1)
        else:
            # Normalize direction
            if direction in ("right", "r"):
                direction = "right"
            elif direction in ("left", "l"):
                direction = "left"
            else:
                print("Unknown direction:", direction)
                print("Choose either l (left) or r (right).")
                sys.exit(1)
            # Set output for built-in display
            cmd = Command("xrandr")
            cmd += "--output " + str(self._builtIn)
            cmd += "--auto"
            cmd += "--rotate normal"
            cmd += "--pos 0x0"
            # Set output for remaining display
            prev = self._builtIn
            for disp in (x for x in self._displays if x != self._builtIn):
                cmd += "--output " + str(disp)
                cmd += "--auto"
                cmd += "--rotate normal"
                cmd += "--" + direction + "-of " + str(prev)
                prev = disp
            cmd.call()
            self.print_status("Extended output.")

    def list_displays(self):
        """List connected displays and their available resolutions."""
        for display in self._displays:
            print(display)
            for resolution in display.get_resolutions():
                print("  " + str(resolution))

    def print_help(self):
        """List available options."""
        print("  -c, --clone\tClone output to external display")
        print("  -e, --extend")
        print("\tleft\tExtend output to the left")
        print("\tright\tExtend output to the right")
        print("  -s, --solo\tReduce output to primary display")
        print("  -l, --list\tList available displays")

def main():
    """Parse arguments and call the respective methods."""
    c = Controller()
    # Quit program if too few arguments are provided.
    if len(sys.argv) < 2:
        c.print_help()
        sys.exit(1)
    else:
        args = sys.argv[1:]
        mode = args[0]
        # Clone output
        if mode in ("--clone", "-c"):
            c.clone_output()
        # Extend output
        elif mode in ("--extend", "-e"):
            c.extend_output(args[1])
        # Reduce output to single display
        elif mode in ("--solo", "--single", "-s"):
            c.reduce_output(c.get_built_in_display())
        # List displays
        elif mode in ("--list", "-l"):
            c.list_displays()
        elif mode in ("--help", "-h"):
            c.print_help()
        # Unknown command, so quit program
        else:
            print("Unknown option:", mode)
            print("Use option -h (--help) to list available options.")
            sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

try:
    long_description = open("README.rst").read()
except IOError:
    long_description = ""

setup(
    name="disport",
    version="0.1.0",
    description="A xrandr wrapper",
    license="MIT",
    author="Tim Wunderlich",
    author_email="mail@tim-wunderlich.de",
    url="https://github.com/TimWunderlich/disport",
    packages=find_packages(),
    install_requires=[],
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],
    entry_points = {
        "console_scripts": ['disport = disport.disport:main']
        },
)



# -*- coding: utf-8 -*-

import subprocess


class Command:
    """Stores shell command as a string.

    Provides the following methods:
        1) call(feedback=False)

    Provides the following operators:
        1) str(cmd)
        2) cmd + str
        3) cmd += str
    """
    def __init__(self, cmd=""):
        """Applies initial command to private member."""
        self._cmd = cmd

    def call(self, feedback=False):
        """Calls shell command and my return shell output."""
        # Split string into an array to be process by subprocess
        cmd = self._cmd.strip().split(" ")
        if feedback:
            # Call command and convert byte array to string
            return subprocess.check_output(cmd).decode("utf-8")
        else:
            # Just call the command
            subprocess.call(cmd)
            return None

    def __str__(self):
        """Return command as a string."""
        return self._cmd

    def __radd__(self, other):
        """Append string to command and return the result."""
        try:
            return self._cmd + " " + str(other)
        except:
            return NotImplemented

    def __iadd__(self, other):
        """Append string to command, store the result and return it."""
        try:
            self._cmd += " " + str(other)
            return self
        except:
            return NotImplemented



# -*- coding: utf-8 -*-

import sys

class Resolution:
    """Stores width and height of a certain resolution.

    Provides the following methods:
        1) get_x()
        2) get_y()

    Provides the following operators:
        1) str(resolution)
        2) resolution1 < resolution2
        3) resolution1 > resolution2
        4) resolution1 == resolution2
        5) resolution1 != resolution2
        5) resolution1 <= resolution2
        5) resolution1 >= resolution2
    """
    def __init__(self, resolution):
        """Assume that the resolution is provided as a string
        in the format WxH.
        """
        # Split at 'x' and store dimensions as integers.
        try:
            x, y = resolution.strip().split("x")
            self._x = int(x)
            self._y = int(y)
        
        # If not possible, set to 0.
        except ValueError:
            self._x = 0
            self._y = 0

    def get_x(self):
        """Return width."""
        return self._x

    def get_y(self):
        """Return height."""
        return self._y

    def __str__(self):
        """Return resolution in the format WxH,
        so xrandr can process it.
        """
        return "x".join((str(self._x), str(self._y)))

    def __lt__(self, other):
        """Check if own dimenions are smaller
        than the dimenions of another resolution.
        If it’s not clear by the widths, compare the heights.
        """
        if isinstance(other, Resolution):
            if self._x < other.get_x():
                return True
            elif self._x == other.get_x():
                return self._y < other.get_y()
            else:
                return False
        else:
            return NotImplemented

    def __gt__(self, other):
        """Check if own dimenions are bigger
        than the dimenions of another resolution.
        If it’s not clear by the widths, compare the heights.
        """
        if isinstance(other, Resolution):
            if self._x > other.get_x():
                return True
            elif self._x == other.get_x():
                return self._y > other.get_y()
            else:
                return False
        else:
            return NotImplemented

    def __eq__(self, other):
        """Check if own dimensions are equal to another resolution."""
        if isinstance(other, Resolution):
            return self._x == other.get_x() and self._y == other.get_y()
        else:
            return NotImplemented

    def __nq__(self, other):
        """Check if own dimenions are unequal to another resolution."""
        if isinstance(other, Resolution):
            return not(self == other)
        else:
            return NotImplemented

    def __le__(self, other):
        """Check if own dimenions are smaller
        than or equal to the dimenions of another resolution.
        """
        if isinstance(other, Resolution):
            return self < other or self == other
        else:
            return NotImplemented

    def __ge__(self, other):
        """Check if own dimenions are smaller
        than or equal to the dimenions of another resolution.
        """
        if isinstance(other, Resolution):
            return self > other or self == other
        else:
            return NotImplemented


class Display:
    """Stores name and resolutions of a display.

    Provides the following methods:

        1) get_resolutions()

    Provides the following operators:

        1) str(display)
        2) resolution in display
        3) display1 & display2
        4) display1 == display2
        5) display1 != display2
    """
    def __init__(self, name, resolutions):
        """Store name and resolutions."""
        # Apply name
        try:
            self._name = str(name)
        except ValueError:
            print("Could not apply name to display.")
            sys.exit(1)

        # Apply all legitimate resolutions.
        self._resolutions = (res for res in resolutions if res.get_x() > 0)

    def get_resolutions(self):
        """Returns list of available resolutions."""
        return self._resolutions

    def __str__(self):
        """Returns the name of the display as a string."""
        return self._name

    def __contains__(self, resolution):
        """Returns True if resolution is available to display."""
        if isinstance(resolution, Resolution):
            return resolution in self._resolutions
        else:
            return NotImplemented

    def __and__(self, other):
        """Returns list of resolutions shared
        between display1 and display2.
        """
        if isinstance(other, Display):
            return (res for res in self._resolutions if res in other)
        else:
            return NotImplemented

    def __eq__(self, other):
        """Returns True if the names of
        display1 and display2 are identical.
        """
        if isinstance(other, Display) or isinstance(other, str):
            return self._name == str(other)
        else:
            return NotImplemented

    def __nq__(self, other):
        """Returns True if the names of
        display1 and display2 are different.
        """
        if isinstance(other, Display) or isinstance(other, str):
            return not(self == other)
        else:
            return NotImplemented
